<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(BASEPATH.'libraries/Model.php');
require_once(APPPATH.'classes/dataobjects/DataObjectBase.class.php');	
/**
 * @abstract
 */
abstract class DatabaseTableModelBase extends Model 
{
	/**
	 * @var テーブル名
	 * @abstract
	 */
	protected $_tablename = '';
	/**
	 * @var PKEY を表すフィールド名
	 * @abstract
	 */
	protected $_pkey_fields = array('ID');
	/**
	 * @var ユニークキー表すフィールド名
	 * @abstract
	 */
	protected $_unique_fields = array();
	/**
	 * @var string データを保持する DataObjectBase 型のクラスの名前
	 * @abstract
	 */
	protected $_object_name = 'DataObjectBase';

	/**
	 * クエリ結果格納
	 */
	protected $_query = null;
	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }
    /**
	 * データオブジェクトを元にレコードにインサートします。
	 * 
	 * @param DataObjectBase $obj データオブジェクト
	 * @return bool 成功時は true
     */
    public function insertObject(DataObjectBase $obj)
    {
    	// オブジェクトを配列に変換
    	$values = $obj->toStringArray();
    	// 配列を元にレコードにインサート
    	return $this->insertArray($values);
    }
    /**
	 * 配列を元にレコードにインサートします。
	 * 
	 * @param array $values データの連想配列
	 * @return bool 成功時は true
     */
    public function insertArray(array $values)
    {
    	// INSERT
    	$res = $this->db->insert($this->_tablename, $values);
    	// 正常なら true で復帰
    	return ($res !== false);
    }
    /**
	 * オブジェクトを元にレコードをアップデートします。
	 * 
	 * @param DataObjectBase $obj データオブジェクト
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function updateObject(DataObjectBase $obj, $keys = array())
    {
    	// オブジェクトを配列に変換
    	$values = $obj->toStringArray();
    	// 配列を元にレコードをアップデート
    	return $this->updateArray($values, $keys);
    }
    /**
	 * 配列を元にレコードをアップデートします。
	 * 
	 * @param array $values データの連想配列
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function updateArray(array $values, array $keys = array())
    {
    	// 検索条件配列の作成 
    	$where = $this->_makeCondition($values, $keys);
    	// UPDATE
    	$res = $this->db->update($this->_tablename, $values, $where);
    	// 正常なら true で復帰
    	return ($res !== false);
    }
    /**
	 * オブジェクトを元に、既にレコードが存在すれば更新、存在しなければ挿入します。
	 * 
	 * @param DataObjectBase $obj データオブジェクト
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function replaceObject(DataObjectBase $obj, array $keys = array())
    {
    	// オブジェクトを配列に変換
    	$values = $obj->toStringArray();
    	// 配列を元にレコードをアップデート
    	return $this->replaceArray($values, $keys);
    }
    /**
	 * 配列を元に、既にレコードが存在すれば更新、存在しなければ挿入します。
	 * 
	 * @param array $values データの連想配列
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function replaceArray(array $values, array $keys = array())
    {
		// 既存チェック
		$res = true;
		if($this->isExistArray($values, $keys))
			$res = $this->updateArray($values, $keys);
		else
			$res = $this->insertArray($values);
    	//
    	return ($res !== false);
    }
    /**
	 * オブジェクトを元にレコードを削除します。
	 * 
	 * @param DataObjectBase $obj データオブジェクト
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function deleteObject(DataObjectBase $obj, array $keys = array())
    {
    	// オブジェクトを配列に変換
    	$values = $obj->toStringArray();
    	// 配列を元にレコードをアップデート
    	return $this->deleteArray($values, $keys);
    }
    /**
	 * 配列を元にレコードを削除します。
	 * 
	 * @param array $values データの連想配列
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 成功時は true
     */
    public function deleteArray(array $values, array $keys = array())
    {
    	// 検索条件配列の作成 
    	$where = $this->_makeCondition($values, $keys);
    	// UPDATE
    	$res = $this->db->delete($this->_tablename, $where);
    	// 正常なら true で復帰
    	return ($res !== false);
    }
    /**
     * 指定オブジェクトと同じキーを持つレコードが存在するかどうかを返します
	 * 
	 * @param DataObjectBase $obj データオブジェクト
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 存在する場合は true
     */
    public function isExistObject(DataObjectBase $obj, array $keys = array())
    {
    	// オブジェクトを配列に変換
    	$values = $obj->toStringArray();
    	//
    	return $this->isExistArray($obj, $keys);
    }
    /**
     * 指定配列と同じキーを持つレコードが存在するかどうかを返します
	 * 
	 * @param array $values データの連想配列
	 * @param array $keys 更新条件に使うフィールド名の配列。省略時は既定のユニークフィールド。
	 * @return bool 存在する場合は true
     */
    public function isExistArray(array $values, array $keys = array())
    {
    	// 検索条件配列の作成 
    	$where = $this->_makeCondition($values, $keys);
    	// レコードの件数を取得
    	$count = $this->getCount($where);
		// count があれば true で復帰
		return ($count > 0);    	
    }
    /**
     * 条件に合うレコードの件数を返します
	 * 
	 * @param array $where 更新条件の連想配列。省略時は条件無し。
	 * @return integer 件数
     */
    public function getCount(array $where = array()) 
    {
    	// SELECT
    	$this->db->select('count(*) AS "count"', false);
    	$query = $this->db->get_where($this->_tablename, $where);
    	// クエリ異常なら null で復帰
    	if(!$query)
    		return null;
    	// 件数0なら 0 で復帰
    	if($query->num_rows() <= 0)
    		return 0;
		// count があれば その値 で復帰
		$row = $query->row();
		return $row->count;
	}
	/**
	 * 検索条件配列を作成します
	 * 
	 * @param array $values 値の連想配列
	 * @param array $keys 検索に使用するフィールド名の配列
	 * @return array 検索条件として使用できる連想配列
	 */
	protected function _makeCondition(array $values, array $keys = array()) 
	{
    	$where = null;
    	// インデックスフィールドを設定 
    	if(empty($keys))
    		$keys = $this->_unique_fields;
    	// 条件配列構築
    	foreach($keys as $key)
    		$where[$key] = $values[$key];
    	//
    	return $where;
	}
//	/**
//	 * データ配列をデータオブジェクトに変換して返します
//	 * 
//	 * @param array $datas メンバー名=>値 形式の連想配列
//	 * @return DataObjectBase データオブジェクト
//	 */
//    protected function _convArrayToObject(array $row)
//    {
//    	if(empty($row))
//    		return null;
//    	return $this->_createRowObject($row);
//    }
    /**
	 * データオブジェクトのインスタンス作成
	 * 
	 * @abstract
	 * @param array $datas メンバー名=>値 形式の連想配列
	 * @return DataObjectBase データオブジェクトのインスタンス
     */
    public function createRowObject($datas = array()) 
    {
		return new $this->_object_name($datas);
	}
	/**
	 * レコードを検索して返します
	 */
    public function getRow(array $where = array())
    {
    	// 
    	if(!$this->fetchFirst($where))
    		return false;
		// 結果復帰
		return $this->_query->row_array();    	
	}
	/**
	 * レコードを検索して返します
	 */
    public function getRowArray(array $where = array(), $limit = null, $offset = null, array $order = array())
    {
    	// 
    	if(!$this->fetchFirst($where, $limit, $offset, $order))
    		return false;
		// 結果復帰
		return $this->_query->result_array();    	
	}
	/**
	 * レコードを検索して返します
	 */
    public function fetchFirst(array $where = array(), $limit = null, $offset = null, array $order = array())
    {
    	// ショートカット
    	$DB =& $this->db;
		$this->freeQueryResult();
		// ORDER BY
		foreach($order as $key => $val)
			$DB->order_by($key, $val);
    	// 実行
		$this->_query = $DB->get_where($this->_tablename, $where, $limit, $offset);
    	// クエリ異常か件数0なら false で復帰
    	if(!$this->_query || $this->_query->num_rows() <= 0)
    		return false;
		// 結果復帰
		return $this->_query;    	
	}
	/**
	 * テーブルの行数を返します。
	 * 
	 * @return integer テーブルの行数
	 */
    public function getRowsAllCount()
    {
		return $this->db->count_all($this->_tablename);
	}
	/**
	 * 先頭行のデータを返します。
	 * 実行後はカレント位置が取得した行に移ります。
	 * 取得できなかった場合は null を返します。
	 * 
	 * @return array 取得した配列データ。取得できなかった場合は null
	 */
    public function firstRow()
    {
    	//
		if(@$this->_query->_data_seek(0))
			return $this->nextRow();
		else
			return null;
	}
	/**
	 * 最終行のデータを返します。
	 * 実行後はカレント位置が取得した行に移ります。
	 * 取得できなかった場合は null を返します。
	 * 
	 * @return array 取得した配列データ。取得できなかった場合は null
	 */
    public function lastRow()
    {
		//
		$maxrow = $this->_query->num_rows();
		if($maxrow)
		{
			@$this->_query->_data_seek($maxrow-1);
			return $this->previousRow();
		}
		else
			return null;
	}
	/**
	 * 現在行の一つ後のデータを返します。
	 * 実行後はカレント位置が取得した行に移ります。
	 * 実行前の現在行が最終だった場合は、カレント位置は失われ null を返します。
	 * 
	 * @return array 取得した配列データ。取得できなかった場合は null
	 */
    public function nextRow()
    {
    	//
		$res = $this->_query->_fetch_assoc();
		if($res)
			$this->_query->current_row++;
		else
			$this->_query->current_row = -1;
		//
		return empty($res)? null : $res;
//		return empty($res)? null : $this->convRowObject($res);
	}
	/**
	 * 現在行の一つ前のデータを返します。
	 * 実行後はカレント位置が取得した行に移ります。
	 * 実行前の現在行が先頭だった場合は、カレント位置は失われ null を返します。
	 * 
	 * @return array 取得した配列データ。取得できなかった場合は null
	 */
    public function previousRow()
    {
    	//
		$res = $this->_query->_fetch_assoc();
		if($res)
		{
			$this->_query->current_row--;
			@$this->_query->_data_seek($this->current_row);
		}
		else
			$this->_query->current_row = -1;
		//
		return empty($res)? null : $res;
//		return empty($res)? null : $this->convRowObject($res);
	}
	/**
	 * 直前の insert した id を返します
	 * 
	 * @return integer insert した id
	 */
    public function getInsertId()
    {
		return $this->db->insert_id();
	}
	/**
	 * クエリ結果の行数を返します
	 * 
	 * @return integer クエリ結果の行数
	 */
    public function getQueryRowsCount()
    {
		return $this->_query->num_rows();
	}
	/**
	 * クエリ結果の連想配列を返します
	 * 
	 * @return array[] クエリ結果の連想配列
	 */
    public function getQueryResult()
    {
		return $this->_query->result_array();
	}
	/**
	 * クエリ結果を解放します
	 * 
	 * @return void
	 */
    public function freeQueryResult()
    {
		if(empty($this->_query))
			return;
    	$this->_query->free_result();
    	$this->_query = null;
	}
}

/* End of file */