<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(APPPATH.'classes/ci_extend_libraries/DatabaseTableModelBase.abstract.php');
	
/**
 * GenericMstModel クラス
 */
class GenericMstModel extends DatabaseTableModelBase 
{
	/**
	 * テーブル名
	 * @abstract
	 */
	protected $_tablename = 'generic_mst';
	/**
	 * PKEY を表すフィールド名
	 * @abstract
	 */
	protected $_pkey_fields = array('ID');
	/**
	 * ユニークキー表すフィールド名
	 * @abstract
	 */
	protected $_unique_fields = array('SECTION', 'ENTRY');

	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }
    /**
     * 指定セッションの指定エントリを返します。
     * エントリを省略した場合はセッション内の全てのレコードを返します
	 * 
	 * @param string $secion セクション名
	 * @param string $entry エントリー名。省略時はセクション内の全エントリー
	 * @param bool $isarray true なら結果を連想配列で返す。
	 * @param bool $asc $entry 省略時、true ならエントリー名で昇順ソートして返す
	 * @param integer $limit $entry 省略時、返すエントリーの個数を指定する
	 * @param integer $offset $entry 省略時、返すエントリーのオフセットを指定する
	 * @return string|array 検索結果のエントリーの値 
     */
    public function getEntry($secion, $entry = null, $isarray = false, $asc = true, $limit = null, $offset = null)
    {
    	// ショートカット
    	$DB =& $this->db;
		$this->freeQueryResult();
		//
		$where = array();
		$where['SECTION'] = $secion;
		if($entry)
			$where['ENTRY'] = $entry;
    	// ORDER BY
		$orderstr = $asc? ' asc' : ' desc';
    	$DB->order_by('ENTRY', $orderstr);
    	// 実行
		$this->_query = $DB->get_where($this->_tablename, $where, $limit, $offset);
		$query =& $this->_query;
		//
		if($query === false || $query->num_rows() <= 0)
			return null;

		// 結果セットの作成 
		$result = null;
		if(is_null($entry))
		{
			// セクション全てを配列で作成
			if($isarray)
				$result = $query->result_array();
			else
			{
				foreach($query->result_array() as $key => $value)
					$result[$value['ENTRY']] = $value['VALUE'];
			}
		}
		else
		{
			// 一つのエントリ
			$result = $query->row_array();
			if(!$isarray)
				$result = $result['VALUE'];
		}
		//
		return $result;
	}
    /**
     * 指定セッションの指定エントリに値を設定します。
	 * 
	 * @param string $secion セクション名
	 * @param string $entry エントリー名。
	 * @param string $value 登録する値。
	 * @param string $value2 登録する値２。
	 * @return bool 成功なら true 
     */
    function setEntry($secion, $entry, $value, $value2 = null)
    {
    	// ショートカット
    	$DB =& $this->db;
		$this->freeQueryResult();
		// 配列作成
		$values = array(
			'SECTION'	=> $secion,
			'ENTRY'		=> $entry,
			'VALUE'		=> $value,
			'VALUE2'	=> $value2,
		);
    	// 配列を元にレコードをアップデート
    	return $this->replaceArray($values);
	}
	/**
     * 指定セッションの指定エントリを削除します。
     * エントリを省略した場合はセッション内の全てのレコードを削除します
     * 
	 * @param string $secion セクション名
	 * @param string $entry エントリー名。省略した場合はセッション内の全てのレコード
	 * @return bool 成功なら true 
	 */
    function unsetEntry($secion, $entry = null)
    {
    	// ショートカット
    	$DB =& $this->db;
		$this->freeQueryResult();
		// WHERE
		$where = array();
		$where['SECTION'] = $secion;
		if($entry)
			$where['ENTRY'] = $entry;
    	// 実行
    	return $this->deleteArray($where, array_keys($where));
	}
	/**
	 * 指定セッションのエントリー数を返します
	 * 
	 * @param string $secion セクション名
	 * @return integer エントリーの数 
	 */
	function getEntryCount($secion) 
	{
    	// ショートカット
    	$DB =& $this->db;
		$this->freeQueryResult();
    	// 件数取得
    	$where = array(
			'SECTION'	=> $secion,
    	);
    	return $this->getCount($where);
	}
}

/* End of file */