<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(APPPATH.'classes/ci_extend_libraries/PaymentDriverBase.abstract.php');
require_once('PaypalSubmitData.class.php');
require_once('PaypalResponseData.class.php');

/**
 * paypal.driver クラス
 */
class PaypalPaymentDriver extends PaymentDriverBase
{
	/**
	 * 
	 */
	protected $_submit_object_name = 'PaypalSubmitData';
	protected $_response_object_name = 'PaypalResponseData';
	/**
	 * 
	 */
//    protected $_user_agent = "Mozilla/4.0 (compatible; MSIE 7.00; Windows NT 5.2)";
	/**
	 * コンストラクタ
	 */
    public function __construct()
    {
        parent::__construct();
    }

	/**
	 * sale transaction
	 * 
	 * @return bool 成功時は true
	 */
	public function transactionSale($paramobj, $transid = null)
	{
		return $this->_submitPayment('S', $paramobj, $transid);
	}
   	/**
     * authorization transaction
     */
    public function transactionAuthorization($paramobj, $transid = null)
	{
		return $this->_submitPayment('A', $paramobj, $transid);
	}
    /**
     * delayed capture transaction
     */
    public function transactionDelayedCapture($paramobj, $transid = null)
	{
		return $this->_submitPayment('D', $paramobj, $transid);
	}
	/**
	 * credit transaction
	 */
    public function transactionCredit($paramobj, $transid = null) 
	{
		return $this->_submitPayment('C', $paramobj, $transid);
	}
	/**
	 * void transaction
	 */
    public function transactionVoid($paramobj, $transid = null)
	{
		return $this->_submitPayment('V', $paramobj, $transid);
	}
	/**
     * inquiry transaction
	 */
    public function transactionInquiry($paramobj, $transid = null)
	{
		return $this->_submitPayment('I', $paramobj, $transid);
	}
	/**
	 * void or credit transaction
	 */
    public function transactionUnsettled($paramobj, $transid = null)
	{
		// void 可能か確認
		$dovoid = $this->_isAvailableVoid($paramobj, $transid);
		$docredit = !$dovoid;
		// void 実行
		if($dovoid)
		{
			$res = $this->transactionVoid($paramobj, $transid);
			$obj_res =& $this->getResponseObject();
			$docredit = ($this->RESULT != 0);	// 失敗したら credit も実行
		}
		// credit 実行
		if($docredit)
			$res = $this->transactionCredit($paramobj, $transid);
		// 復帰
		return $res;
	}
	/**
	 * 
	 */
    public function isAvailableVoid($paramobj, $transid = null)
	{
		$this->transactionInquiry($paramobj, $transid);
		$obj_res =& $this->getResponseObject();
		return $obj_res->isAvailableVoid();
	}
	/**
	 * 
	 */
    public function isTransactionSucceed()
	{
		$obj_res =& $this->getResponseObject();
		return $obj_res->isSucceed();
	}
    /**
     * 
     */
    protected function _submitPayment($trxtype, $paramobj, $transid = null)
    {
		// サブミット用オブジェクトを取得
		$obj_smt =& $this->getSubmitObject();
		// パラメータをサブミットオブジェクトにセット
		$obj_smt->parseArray($paramobj->toArray());
		// 専用パラメータ設定
		$obj_smt->TRXTYPE = $trxtype;
		$obj_smt->TENDER = 'C';
		$obj_smt->VERBOSITY = 'MEDIUM';
		// サブミット用オブジェクトからパラメータ文字列の取得 
		$namevalue = $obj_smt->toNamevalueString(false, array_values($obj_smt->getPropertyNames()));	// TODO2: ここで array_values はきたない
		// トランザクションIDをインスタンスに設定（新規の場合は新IDを取得）
		$newtransid = $this->setTransactionId($transid);
		// サブミット実行
		$response  = $this->_execCurl($namevalue, $newtransid, $this->_payment_info->SUBMIT_URL);
		// インスタンスにサブミット実行情報を保存
		$this->setSubmitValue($namevalue);
		$this->setResponseValue($response);
		// レスポンス用オブジェクトを取得
		$obj_rps =& $this->getResponseObject();
		// レスポンス文字列からレスポンスオブジェクトを初期化＆復帰
		$resvalues = strstr($response, 'RESULT');
		if($resvalues)
		{
			$obj_rps->parseNamevalueString($resvalues);
			return true;
		}
		else
			return false;
    }
    /**
     * 
     */
    public function initialize($merchant, $payment)
    {
    	parent::initialize($merchant, $payment);
    	//
		$obj_smt =& $this->getSubmitObject();
		$obj_smt->USER		= $payment->ACCOUNT;
		$obj_smt->VENDOR	= $payment->ACCOUNT_EX1;
		$obj_smt->PARTNER	= $payment->ACCOUNT_EX2;
		$obj_smt->PWD		= $payment->PASSWORD;
    }
    /**
     * リクエストヘッダを作成して返します
     */
    protected function _makeCurlHeaders($transid)
    {
    	//
		$curl_headers = array(
			'Content-Type: text/namevalue',
			'X-VPS-Timeout: 30',
			'X-VPS-VIT-OS-Name: Linux',
			'X-VPS-VIT-OS-Version: RHEL 5.2',
			'X-VPS-VIT-Client-Type: PHP/cURL',
			'X-VPS-VIT-Client-Version: 0.01',
			'X-VPS-VIT-Client-Architecture: x86',
			'X-VPS-VIT-Integration-Product: PaymentGW',
			'X-VPS-VIT-Integration-Version: 0.01',
		);
    	$curl_headers[] = "X-VPS-Request-ID: $transid";
    	//
    	return $curl_headers;
    }
    /**
     * リクエストを発行します
     * 
     * @param string $namevalue 送信するパラメータリスト
     * @param string $transid 内部生成したトランザクションID
     * @param string $submit_url 送信する URL
     * @param string $user_agent ユーザエージェント。省略時は送信しない
     * @return string 発行先から戻されたレスポンス文字列
     */
    protected function _execCurl($namevalue, $transid, $submit_url/*, $user_agent = null*/)
    {
    	//
		$ch = curl_init();
		//
		$submit_url = $this->_payment_info->SUBMIT_URL;
		$transid = $this->getTransactionId();
		$headers = $this->_makeCurlHeaders($transid);
		// 
		curl_setopt($ch, CURLOPT_URL, $submit_url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
//		curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 45);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $namevalue);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST,  2);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, true);
		curl_setopt($ch, CURLOPT_POST, 1);
		//
		$res = curl_exec($ch);
//		$res  = 'RESULT=0&RESPMSG=GEMA&PNREF=1234567890';
		//
		curl_close($ch);
		//
		return $res;
    }
}

/* End of file */